package com.example.facetter;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;

import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.Bitmap.CompressFormat;
import android.os.Environment;
import android.text.format.DateFormat;
import android.util.Log;

public class ImageManager {
    private static final String TAG = "ImageManager";

    private static String createName(long dateTaken) {
        return DateFormat.format("yyyy-MM-dd-kk-mm-ss", dateTaken).toString();
    }

    public static String addImageAsApplication(Context context, Bitmap bitmap) {
        long dateTaken = System.currentTimeMillis();
        String name = createName(dateTaken) + ".jpg";
        // アプリケーション名を取得
        String appName = context.getResources().getString(R.string.app_name);
        // SDカードのディレクトリ/アプリケーション名のディレクトリパスを作成
        String path = Environment.getExternalStorageDirectory().toString() + "/" + appName;
        return addImageAsApplication(context, name, dateTaken, path, name, bitmap, null);
    }

    public static String addImageAsApplication(Context context, String name,
            long dateTaken, String directory,
            String filename, Bitmap source, byte[] jpegData) {

        OutputStream outputStream = null;
        String filePath = directory + "/" + filename;
        try {
            // ディレクトリがなければ作成
            File dir = new File(directory);
            if (!dir.exists()) {
                dir.mkdirs();
                Log.d(TAG, dir.toString() + " create");
            }
            // ファイル出力
            File file = new File(directory, filename);
            if (file.createNewFile()) {
                outputStream = new FileOutputStream(file);
                if (source != null) {
                    // JPEG圧縮してファイル出力
                    source.compress(CompressFormat.JPEG, 75, outputStream);
                } else {
                    outputStream.write(jpegData);
                }
            }

        } catch (FileNotFoundException ex) {
            Log.w(TAG, ex);
            return null;
        } catch (IOException ex) {
            Log.w(TAG, ex);
            return null;
        } finally {
            if (outputStream != null) {
                try {
                    outputStream.close();
                } catch (Throwable t) {
                }
            }
        }

        // ギャラリーへ反映
        new MediaScannerNotifier(context, filePath, "image/jpeg");
        return filePath;
    }
}
