package org.siprop.android;

import java.io.IOException;
import java.io.InputStream;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.HashMap;
import java.util.Map;

import org.siprop.android.location.LocationInfo;
import org.siprop.android.overlay.MapViewOverlay;
import org.siprop.android.overlay.TweetItemizedOverlay;
import org.siprop.android.overlay.TweetOverlayItem;
import org.siprop.android.overlay.UserItemizedOverlay;
import org.siprop.android.overlay.UserOverlayItem;
import org.siprop.android.util.TwitterToken;

import twitter4j.AsyncTwitter;
import twitter4j.AsyncTwitterFactory;
import twitter4j.GeoLocation;
import twitter4j.Query;
import twitter4j.QueryResult;
import twitter4j.Status;
import twitter4j.Tweet;
import twitter4j.TwitterAdapter;
import twitter4j.TwitterException;
import twitter4j.TwitterListener;
import twitter4j.TwitterMethod;
import twitter4j.http.AccessToken;

import com.google.android.maps.GeoPoint;
import com.google.android.maps.MapActivity;
import com.google.android.maps.MapController;
import com.google.android.maps.MapView;

import android.app.Activity;
import android.app.AlertDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.graphics.BitmapFactory;
import android.graphics.drawable.BitmapDrawable;
import android.location.Location;
import android.location.LocationListener;
import android.location.LocationManager;
import android.location.LocationProvider;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.widget.EditText;
import android.widget.Toast;

/**
 * メインのアクティビティ
 * 
 * @author hirotaka@siprop.org
 *
 */
public class TwitLalo extends MapActivity implements LocationListener {
	
	// Map用設定
	MapView mapView ;
	MapViewOverlay mapViewOverlay;
	MapController mapController;
	LocationManager locationManager;

	UserItemizedOverlay itemizedOverlay;
	TweetItemizedOverlay tweetItemizedOverlay;
	Activity twitLaloActivity = this;

	LocationInfo ownLocationInfo = new LocationInfo();
	
	// Twitter用
	TwitterListener twitterListener;
	Boolean firstStart = false;
	AsyncTwitter asyncTwitter;
	private final static int SEARCH_DIST = 10;
	View settingView;
	
	// ユーザイメージ画像リスト
	Map <Integer, BitmapDrawable> userImage = new HashMap<Integer, BitmapDrawable>();
	
	private final static int MESSAGE_TEXT = 0;
	private final static int TWEET_TEXT = 1;
	private final static String TWITLALO_HASH_TAG = "#twitlalo";

	@Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.main);
        
        // 初期処理
        mapView = (MapView) this.findViewById(R.id.mapview);
        mapView.setEnabled(true);
        mapView.invalidate();
        mapViewOverlay = new MapViewOverlay();
        mapView.getOverlays().add(mapViewOverlay);
        locationManager = (LocationManager) getSystemService(Context.LOCATION_SERVICE);
        locationManager.requestLocationUpdates(LocationManager.GPS_PROVIDER, 30000, 5, this);

        // 拡大・縮尺とズームだけ設定しておく
        mapView.setBuiltInZoomControls(true);
        mapController = mapView.getController();
        mapController.setZoom(14);
        
        // オーバーレイ用アイテム
        itemizedOverlay =
        	new UserItemizedOverlay(getResources().getDrawable(
        			R.drawable.twitter));
        tweetItemizedOverlay = 
        	new TweetItemizedOverlay(getResources().getDrawable(
        			R.drawable.pin), this);
        mapView.getOverlays().add(itemizedOverlay);
        mapView.getOverlays().add(tweetItemizedOverlay);
        
        // twitter用非同期処理
        System.setProperty("twitter4j.oauth.consumerKey", getString(R.string.consumerKey)) ;
        System.setProperty("twitter4j.oauth.consumerSecret", getString(R.string.consumerSecret)) ;
        this.twitterListener = new TwitterAdapter() {
        	
            @Override
            public void updatedStatus(Status status) {
	        	Toast.makeText(twitLaloActivity, "投稿が完了しました。メニューから近くのツイートを検索して下さい。", Toast.LENGTH_LONG).show();
            }

            @Override
            public void retweetedStatus(Status status) {
	        	Toast.makeText(twitLaloActivity, "投稿が完了しました。メニューから近くのツイートを検索して下さい。", Toast.LENGTH_LONG).show();
            }

            @Override
            public void searched(QueryResult result) {
				for (Tweet tweet : result.getTweets()) {
					if (tweet.getGeoLocation() != null) {
						if (userImage.get(tweet.getFromUserId()) == null) {
							try {
								URL url = new URL(tweet.getProfileImageUrl());
			    	            InputStream is = url.openStream();
			    	            userImage.put(tweet.getFromUserId(),
			    	            		new BitmapDrawable(BitmapFactory.decodeStream(is)));
							} catch (MalformedURLException e) {
								e.printStackTrace();
							} catch (IOException e) {
								e.printStackTrace();
							}
						}
						
						GeoPoint geoPoint = new GeoPoint(
	    		        		(int)(tweet.getGeoLocation().getLatitude() * 1E6),
	    		        		(int)(tweet.getGeoLocation().getLongitude() * 1e6));
						TweetOverlayItem tweetOverlayItem = new TweetOverlayItem(
								geoPoint, "", "", tweet);
						if (userImage.get(tweet.getFromUserId()) != null)
							tweetItemizedOverlay.addOverlay(tweetOverlayItem,
									userImage.get(tweet.getFromUserId()));
						else
							tweetItemizedOverlay.addOverlay(tweetOverlayItem,
									getResources().getDrawable(R.drawable.pin));
					}
				}
            }
            
            @Override
            public void onException(TwitterException exception, TwitterMethod method) {
                if (method == TwitterMethod.UPDATE_STATUS || method == TwitterMethod.RETWEET_STATUS) {
                } else if (method == TwitterMethod.SEARCH ) {
    	        	Toast.makeText(twitLaloActivity, "検索が失敗しました...", Toast.LENGTH_LONG).show();
                } else {
                	throw new AssertionError("Should not happen");
                }
            }
        };
        
        
        // Twitter用処理
        if (TwitterToken.getToken() == null || TwitterToken.getTokenSecret() == null) 
        	showIdPass();
        else {
    	    AsyncTwitterFactory factory = new AsyncTwitterFactory(twitterListener);
    		asyncTwitter = factory.getOAuthAuthorizedInstance(
    				new AccessToken(TwitterToken.getToken(), TwitterToken.getTokenSecret()));
    		try {
				ownLocationInfo.setUserId(asyncTwitter.getId());
				searchNearTweet();
			} catch (IllegalStateException e) {
				e.printStackTrace();
			} catch (TwitterException e) {
				e.printStackTrace();
			}
        }
    }
    
	@Override
	protected boolean isRouteDisplayed() {
		return false;
	}
	
	/**
	 * ID/Passwordの設定
	 */
	private void showIdPass() {
        LayoutInflater factory = LayoutInflater.from(this);
        settingView = factory.inflate(R.layout.id, null);
        
        AlertDialog.Builder dlg =new AlertDialog.Builder(this); 
        dlg.setMessage("ID/Passwordを設定してください。");
        dlg.setTitle("Twitter Setting");
        dlg.setView(settingView);
      
        dlg .setPositiveButton("OK", new DialogInterface.OnClickListener(){
			public void onClick(DialogInterface dialog, int which) {
				EditText userid =(EditText)settingView.findViewById(R.id.userid);
				EditText password =(EditText )settingView.findViewById(R.id.userpassword);

				if (userid.getText() != null && password.getText() != null) {
		    	    AsyncTwitterFactory factory = new AsyncTwitterFactory(twitterListener);
		    	    asyncTwitter = factory.getInstance(userid.getText().toString(), password.getText().toString());
		    		try {
						AccessToken token = asyncTwitter.getOAuthAccessToken();
						ownLocationInfo.setUserId(asyncTwitter.getId());
						TwitterToken.saveToken(token);
			        	Toast.makeText(twitLaloActivity, "認証成功。メニューから近くのツイートを検索して下さい。", Toast.LENGTH_LONG).show();
						searchNearTweet();
					} catch (TwitterException e) {						
			        	Toast.makeText(twitLaloActivity, "認証失敗。メニューから再度行ってください。", Toast.LENGTH_LONG).show();
					}
		    		
				} else {
					// トースターを開いて認証失敗を告げる...
		        	Toast.makeText(twitLaloActivity, "認証失敗。メニューから再度行ってください。", Toast.LENGTH_LONG).show();
				}
			}
        });
        dlg.create();
        dlg.show();
	}
	
	
	/**
	 * 他のアクティビティからの戻り処理
	 */
    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent intent) { 
    	// メッセージの投稿
    	if (requestCode == MESSAGE_TEXT && resultCode == RESULT_OK) {
            Bundle extras = intent.getExtras();
            if (extras != null) {
            	String message = extras.getString("message");
				GeoPoint geoPoint = mapViewOverlay.getGeoPoint();
				this.asyncTwitter.updateStatus(
						message + " " + TWITLALO_HASH_TAG,
							new GeoLocation(
								geoPoint.getLatitudeE6() / 1E6,
								geoPoint.getLongitudeE6() / 1E6));
            }
        	Toast.makeText(this, "Tweetを投稿しました。メニューから近くを検索して確認してみて下さい。", Toast.LENGTH_LONG).show();

        // retweetとか
    	} else if (requestCode == TWEET_TEXT && resultCode == RESULT_OK) {
            Bundle extras = intent.getExtras();
            if (extras!=null) {
            	String message = extras.getString("message");
            	Tweet tweet = (Tweet) extras.getSerializable("tweet");
            	String type = extras.getString("type");

            	// Replyの場合
            	if (type.equals("REPLY")) {
            		message = "@" + tweet.getFromUser() + " " + message;

            		// ちょっとずらす
            		int lat = (int)(tweet.getGeoLocation().getLatitude() * 1E6);
            		int lon = (int)(tweet.getGeoLocation().getLongitude() * 1E6);
            		lat += 150; lon += 150;
					GeoLocation geoLocation = new GeoLocation(
							lat/1E6,
							lon/1E6);
					this.asyncTwitter.updateStatus(message + " " + TWITLALO_HASH_TAG,
										tweet.getId(),
										geoLocation);
            	
				// retweetの場合
            	} else if (type.equals("RETWEET")) {
            		this.asyncTwitter.retweetStatus(tweet.getId());
            	}
            	
	        	Toast.makeText(this, "Tweetを投稿しました。メニューから近くを検索して確認してみて下さい。", Toast.LENGTH_LONG).show();
            }
    	}
    }

    /**
     * Resumeした時の処理
     */
    @Override
    protected void onResume() {
        if (locationManager != null) {
        	locationManager.requestLocationUpdates(LocationManager.GPS_PROVIDER, 30000, 5, this);
        }
        super.onResume();
    }
    
    /**
     * Pauseした時の処理
     */
    @Override
    protected void onPause() {
        if (locationManager != null) {
        	locationManager.removeUpdates(this);
        }
        super.onPause();
    }

    
    /**
     * メニューボタンを作成する
     */
    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
    	super.onCreateOptionsMenu(menu);
    	MenuInflater inflater = getMenuInflater();
    	inflater.inflate(R.layout.menu, menu);
    	return true;
    }
    
    /**
     * メニューボタン
     */
    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
    	AlertDialog.Builder builder = new AlertDialog.Builder(this);
    	builder.setTitle("設定");

    	switch (item.getItemId()) {
		case R.id.menu_idsetting:
			showIdPass();			
			break;
			
		case R.id.menu_nowplace:
			if (ownLocationInfo.getLongitude() != null &&
					ownLocationInfo.getLatitude() != null) {
				GeoPoint geoPoint = new GeoPoint(
		        		(int)(ownLocationInfo.getLatitude() * 1E6),
		        		(int)(ownLocationInfo.getLongitude() * 1e6));
		        mapController.animateTo(geoPoint);
			}
			break;
			
		case R.id.menu_nowmessage:
			if (this.asyncTwitter != null) {
				Intent messageActivity = new Intent(TwitLalo.this, MessageActivity.class);
				GeoPoint geoPoint = mapViewOverlay.getGeoPoint();
				messageActivity.putExtra("latitude", geoPoint.getLatitudeE6() / 1E6);
				messageActivity.putExtra("longitude", geoPoint.getLongitudeE6() / 1E6);
				startActivityForResult(messageActivity, MESSAGE_TEXT);
			} else {
	        	Toast.makeText(this, "メニューを開いてID・パスワードを設定してください。", Toast.LENGTH_LONG).show();
			}
			break;
			
		case R.id.menu_nearinfo:
			if (this.asyncTwitter != null)
				searchNearTweet();
			break;

		default:
			break;
		}
    	return true;
    }


	/**
	 * 位置情報取得が不能になった時の処理
	 */
	@Override
	public void onProviderDisabled(String provider) {		
	}

	/**
	 * 位置情報取得が可能になった時の処理
	 */
	@Override
	public void onProviderEnabled(String provider) {		
	}
	
    /**
     * 位置情報の変化を取得する
     */
	@Override
	public synchronized void onLocationChanged(Location location) {
		
		// 現在地を更新する
		ownLocationInfo.setLatitude(location.getLatitude());
		ownLocationInfo.setLongitude(location.getLongitude());
		GeoPoint geoPoint = new GeoPoint(
	        		(int)(ownLocationInfo.getLatitude() * 1E6),
	        		(int)(ownLocationInfo.getLongitude() * 1e6));
	        
        // 初回起動時は現在地を表示する
        if (!firstStart) {
        	mapController.animateTo(geoPoint);
        	firstStart = true;
        }

    	if (this.asyncTwitter != null) {
    		
    		// 自身のアイコンを描画
            UserOverlayItem ownOverlayItem = new UserOverlayItem(geoPoint, "", "", ownLocationInfo);
            
            // 地図上に表示
            itemizedOverlay.addOverlay(ownOverlayItem,
            		getResources().getDrawable(R.drawable.twitter));
    	}
	}

	/**
	 * GPSの状態変化を取得する
	 */
	@Override
	public void onStatusChanged(String provider,
			int status,
			Bundle extras) {
        switch (status) {
        case LocationProvider.AVAILABLE:
            break;
        case LocationProvider.OUT_OF_SERVICE:
        	Toast.makeText(this, "GPSサービスが利用できません", Toast.LENGTH_LONG).show();
            break;
        case LocationProvider.TEMPORARILY_UNAVAILABLE:
        	Toast.makeText(this, "GPSデータを取得できません", Toast.LENGTH_LONG).show();
            break;
        }		
	}

	/**
	 * 近くのtweetを検索する
	 */
	public void searchNearTweet() {
		searchNearTweet(0);
	}
	public void searchNearTweet(long sinceId) {
		GeoPoint geoPoint = mapView.getMapCenter();

		try {
			Query query = new Query();
			query.setGeoCode(new GeoLocation(
					geoPoint.getLatitudeE6() / 1E6,
					geoPoint.getLongitudeE6() / 1E6),
					SEARCH_DIST,
					Query.KILOMETERS);
			query.setQuery(TWITLALO_HASH_TAG);
			if (sinceId != 0) query.sinceId(sinceId);
			this.asyncTwitter.search(query);

		} catch (IllegalStateException e) {
        	Toast.makeText(this, "近くのツイートの検索に失敗しました", Toast.LENGTH_LONG).show();
		}
	}
	
	/**
	 * tweetを表示する
	 * @param tweetOverlayItem
	 */
	public void showTweet(TweetOverlayItem tweetOverlayItem) {
		Intent locationActivity = new Intent(this, LocationActivity.class);
		Tweet tweet = tweetOverlayItem.getTweet();
		locationActivity.putExtra("userimage", userImage.get(tweet.getFromUserId()).getBitmap());
		locationActivity.putExtra("tweet", tweet);
		startActivityForResult(locationActivity, TWEET_TEXT);
	}

}